@@ -18,11 +18,11 @@ our @EXPORT_OK = qw(
 	modelsdir
 ); # symbols to export on request (WHY?)
 
-our $VERSION = '0.3';
+our $VERSION = '0.4';
 
 
 BEGIN {
-    $VERSION = '0.3';
+    $VERSION = '0.4';
     if ($] > 5.006) {
         require XSLoader;
         XSLoader::load(__PACKAGE__, $VERSION);
@@ -56,13 +56,20 @@ sub decode {
 
 	if( ! exists($params->{'graphicaldisplayresult'}) || ! defined($m=$params->{'graphicaldisplayresult'}) ){
 		push @params, 0;
-	} else { push @params, $m }
+	} else {
+		# we were asked for displaying graphical output
+		# do we have highgui? was Opencv compiled with that option?
+		if( ($m == 1) && (opencv_has_highgui_xs() == 0) ){
+			print STDERR "decode() : warning, it seems that the current OpenCV installation does not support 'highgui' (which is optional), so parameter 'graphicaldisplayresult=1' will be ignored.\n";
+			push @params, 0;
+		} else { push @params, $m }
+	}
 
 	if( ! exists($params->{'dumpqrimagestofile'}) || ! defined($m=$params->{'dumpqrimagestofile'}) ){
 		push @params, 0;
 	} else { push @params, $m }
 
-	print "calling with these params: @params\n";
+	print "calling with these params: '".join("','", map { defined($_) ? $_ : '<undef>' } @params)."'.\n";
 
 	return decode_xs(
 		$params[0],
@@ -84,7 +91,7 @@ Image::DecodeQR::WeChat - Decode QR code(s) from images using the OpenCV/WeChat
 
 =head1 VERSION
 
-Version 0.3
+Version 0.4
 
 
 =head1 SYNOPSIS
@@ -180,6 +187,9 @@ Here is some code to get you started:
     # pre-trained models location (installed with this module)
     print "my models are in here: ".Image::DecodeQR::WeChat::modelsdir()."\n"
 
+    # returns 1 or 0 when OpenCV was compiled with highgui or not
+    # and supports GUI display like imshow() which displays an image in a window
+    my $has_highgui_support = opencv_has_highgui_xs();
 
 This code calls functions and methods from OpenCV/WeChat library (written in C++)
 for decoding one or more QR codes found embedded in images.
@@ -213,6 +223,8 @@ interface other parts of the OpenCV library:
 
 =item C<modelsdir()>
 
+=item C<opencv_has_highgui_xs()>
+
 =back
 
 
@@ -228,7 +240,7 @@ A CLI script is provided and will be installed by this module. Basic usage is as
 =head1 SUBROUTINES/METHODS
 
 
-=head3 C< Image::DecodeQR::WeChat::decode_xs(infile, modelsdir, outbase, verbosity, graphicaldisplayresult, dumpqrimagestofile) >
+=head3 C< decode_xs(infile, modelsdir, outbase, verbosity, graphicaldisplayresult, dumpqrimagestofile) >
 
 It takes in the filename of an input image which may contain
 one or more QR codes and returns back an ARRAYref of
@@ -283,7 +295,7 @@ boxes. See C<outbase> above.
 =back
 
 
-=head3 C< Image::DecodeQR::WeChat::decode(\%params) >
+=head3 C< decode(\%params) >
 
 
 This is a Perl wrapper to the C<decode_xs()> so that user can specify
@@ -322,16 +334,44 @@ and the QR-code detected highlighted, once for each code detected.
 =back
 
 
-=head3 C< Image::DecodeQR::WeChat::modelsdir() >
+=head3 C< modelsdir() >
 
 It returns the path where the models included in this package have been installed.
 This is useful when you want to use C<decode_xs()> and need to specify
 the C<modelsdir>. Just pass the output of this to C<decode_xs()> as its C<modelsdir> parameter.
 
+=head3 C< opencv_has_highgui_xs() >
+
+It returns 1 or 0 depending on whether current OpenCV installation has
+support for graphical display of images (the C<imshow()> function). This
+affects the option C<graphicaldisplayresult> to L<decode()> and L<decode_xs()>
+which will be ignored if there is no highgui support.
+
+Caveat: checking for whether current OpenCV installation has highgui
+support is very lame, it merely tries to find the include file C<< opencv2/highgui.hpp >>
+in the Include dirs. I have tried several methods (see ```Makefile.PL```), for
+example L<DynaLoader> or L<FFI::CheckLib> can search for symbols in any library
+(e.g. searching for C<imshow()> in C<libopencv_highgui> or  C<libopencv_world>).
+This would have been the most straight-forward way but alas, these are C++ libraries
+and function names are mangled to weird function names like:
+
+    ZN2cv3viz6imshowERKNSt7__cxx1112basic_stringIcSt11char_traitsIcESaIcEEERKNS_11_InputArrayERKNS_5Size_IiEE()
+
+There's an imshow() in there but without a regex symbol-name search
+the symbol can not be detected.
+
+Another take is with L<Devel::CheckLib> which supports compiling code
+snippets when searching for a particular library. This fails because they
+... allow only a C compiler but we need a C++ compiler.
+
+Finally, using L<Inline::CPP> to compile our own snippet is totally in vain
+because of its useless form of running as C<use Inline CPP => ... >. I could
+not find any way of telling it to use specific CFLAGS and LDFLAGS with this
+useless C<use Inline CPP> form.
+
 
 =head1 IMPLEMENTATION DETAILS
 
-
 This code demonstrates how to call OpenCV (modern OpenCV v4) C++ methods using the
 technique suggested by C<Botje @ #perl> in order to avoid all the
 function, macro, data structures name clashes between Perl and OpenCV
@@ -367,22 +407,23 @@ and
     } //extern "C" {
     #endif
 
-This only need happen in the header file: C< wechat_qr_decode_lib.hpp >.
+This only need happen in the header file: C< wechat_qr_decode_lib.hpp >
+and in the XS file where the Perl headers are included.
 
 
 =head1 INSTALLING OpenCV
 
-
 In my case downloading OpenCV using Linux's package
 manager was not successful.It required to add another
 repository which wanted to install its own versions
-of packages I already had.
+of packages I already had. So I prefered to install
+OpenCV from sources. This is the procedure I followed:
 
 =over
 
 =item Download OpenCV sources and also its contributed modules.
 
-=item Extract the sources and changed to the source dir.
+=item Extract the sources and change to the source dir.
 
 =item From within the source dir extract the contrib archive.
 
@@ -392,19 +433,39 @@ of packages I already had.
 C<cmake-gui> and C<ccmake>. The former is a full-gui interface
 to setting the billion C<cmake> variables. Use it
 if you are on a machine which offers a GUI: C<cmake-gui ..>
- . If you are on a remote host possibly over telnet or ssh
+ . If you are on a headless or remote host possibly over telnet or ssh
 then do not despair because c<ccmake> is the CLI, curses-based
-equivalent to C<cmake-gui>,  use it like: C<ccmake ..> .
+equivalent to C<cmake-gui>,  use it like: C<ccmake ..> (from within the build dir).
 
-=item Once on the interface first C<configure>, then check the
+=item Once on either of the cmake GUIs, first do a
+C<configure>, then check the
 list of all variables (you can search on both, for searching
-in the CLI one press C</> and then C<n> for next hit)
+in the CLI, press C<</>> and then C<n> for next hit)
 to suit you and then C<generate>, quit
 and  C<VERBOSE=1 make -j4 all>
 
-=item I guess, variables you want to change are C<OPENCV_EXTRA_MODULES_PATH>
+=item I guess, cmake variables you want to modify
+are C<OPENCV_EXTRA_MODULES_PATH>
 and C<OPENCV_ENABLE_NONFREE> and anything that has to do with C<CNN> or C<DNN>.
-But I only guess.
+If you have CUDA installed and a CUDA-capable GPU then enable CUDA
+(search for CUDA string to find the variable(s)). Also, VTK, Ceres Solver,
+Eigen3, Intel's TBB, CNN, DNN etc. You need to install all these
+additional packages to get OpenCV support with them.
+
+=item I had a problem with compiling OpenCV with a GUI (the C<highgui>)
+on a headless host. So, I just disabled it. That's easy to achieve
+during the above.
+
+=item I have both installed this on a CUDA-capable GPU (with CUDA 10.2 installed)
+host and on a headless remote host with no GPU or basic. CUDA is
+not required for building this module.
+
+Your mileage may vary.
+
+If you are seriously in need of installing
+this module then consider migrating to a serious operating system
+such as Linux first. M$-Windows will always present problems and
+I don't have the patience of keeping track of them ...
 
 =back
 
